library(rflowcyt)
library(odesolve)
library(ICBflow)
setwd(file.path(.path.package("ICBflow"), "flowData"))

par(mfrow=c(2,4))
H=list(NULL)
timept = c(0,4,8,12,16,24,48)
mid1=NULL
mid2=NULL
for (i in 1:7)
{fcs <- read.FCS(paste(timept[i],"h.LMD", sep="") )
fcsdat=fcs@data
dfr=as.data.frame(fcsdat[,c(4,3)])
xdfr=cbind((dfr[,1]+dfr[,2])/2,(dfr[,2]-dfr[,1])/2)
sxdfr=xdfr[xdfr[,2]>(-40),]
H[[i]]=hist(sxdfr[(sxdfr[,1]>130)&(sxdfr[,1]<=490),1],br=seq(130,490,by=5),main=paste(timept[i],"h  ",sep=""),xlim=c(100,500),xlab="DNA") 
dns=H[[i]]$density
x=H[[i]]$mids
max1=max(dns[x<230]);max1
max2=max(dns[x>380]);max2
mid1[i]=x[which(dns==max1)]
mid2[i]=x[which(dns==max2)]
}  ######### ignore the warnings from read.FCS
#H

fderiv12 <- function(t, X, A) {xp=A%*%X; list(xp)}
nG1=3;nS=12;nG2=2;nM=1
NS=nG1+nS+nG2+nM
idelay=2 # k arrow from state 2 to 3 gets the delay upon serum starvation
isync=3  # initially all mass is this state, aphidicolin is a DNA pol inhibitor so really at G1-S boundary
P=rep(0,NS)
P[isync]=.5
P[-isync]=(1-P[isync])/(NS-1);P
X=c(P[1:(NS-1)],1,10,.1,1)  # this leave the balance in the last state

fopt <- function(X) {
P=X[1:(NS-1)]
k=X[NS]
sig=X[NS+1]
kdmod=X[NS+2]
ksmod=X[NS+3]
y0=c(P,1-sum(P))  # last state was arbitrarily picked as the dependent state probability
dA=rep(k,NS)
dA[(nG1+1):(nG1+nS)]=ksmod*dA[(nG1+1):(nG1+nS)]
A=diag(-dA)
for (i in 2:NS) A[i,i-1]=dA[i-1]
A[1,NS]=2*dA[NS]
out1=lsoda(y=y0,times=c(0,4,8),fderiv12, parms=A, rtol=1e-4, atol= 1e-4)
ny0=out1[nrow(out1),2:(NS+1)]
dA[idelay]=kdmod*dA[idelay]
A=diag(-dA)
for (i in 2:NS) A[i,i-1]=dA[i-1]
A[1,NS]=2*dA[NS]
out2=lsoda(y=ny0,times=c(8,12,16,24,48),fderiv12, parms=A, rtol=1e-4, atol= 1e-4)
outs=data.frame(rbind(out1,out2))[-3,];#outs
PP=outs[,2:(NS+1)]
Nt=apply(PP,1,sum)
PP=PP/Nt
G1=apply(PP[1:nG1],1,sum)
S=apply(PP[(nG1+1):(nG1+nS)],1,sum)
G2M=apply(PP[(nG1+nS+1):NS],1,sum)
x=H[[1]]$mids  # these are all the same since br was set in hist
tot=0  # LSQ sum running across time
for (tt in 1:7){
Ys=rep(0,length(x))
for (i in 1:nS) Ys=Ys+PP[tt,nG1+i]*dnorm(x,mean=mid1[tt]+(mid2[tt]-mid1[tt])*i/(nS+1),sd=sig)
Y=G1[tt]*dnorm(x,mean=mid1[tt],sd=sig)+G2M[tt]*dnorm(x,mean=mid2[tt],sd=sig)+Ys
tot=tot+sum((H[[tt]]$density-Y)^2)
}
tot  }  # fopt returns the total SSQ 

Amatup=cbind(diag(rep(1,NS-1)),matrix(rep(0,(NS-1)*4),ncol=4))
Amatlw=-Amatup
Amatmid=cbind(matrix(rep(0,(NS-1)*4),nrow=4),diag(rep(1,4)) )
AmatSum=c(rep(-1,NS-1),rep(0,4))
Amat=rbind(Amatup,Amatmid,Amatlw,AmatSum)
bvec=c(rep(0,NS-1),rep(0,4),rep(-1,NS-1),-1)
opt=constrOptim(X,fopt,grad=NULL,ui=Amat,ci=bvec)
P=opt$par[1:(NS-1)]
k=opt$par[NS]
sig=opt$par[NS+1]
kdmod=opt$par[NS+2]
ksmod=opt$par[NS+3]

# now replot the final fit against the measured densities
y0=c(P,1-sum(P))  
dA=rep(k,NS)
dA[(nG1+1):(nG1+nS)]=ksmod*dA[(nG1+1):(nG1+nS)]
tau1=sum(1/dA)
A=diag(-dA)
for (i in 2:NS) A[i,i-1]=dA[i-1]
A[1,NS]=2*dA[NS]
out1=lsoda(y=y0,times=c(0,4,8),fderiv12, parms=A, rtol=1e-4, atol= 1e-4)
ny0=out1[nrow(out1),2:(NS+1)]
dA[idelay]=kdmod*dA[idelay]
tau2=sum(1/dA)
A=diag(-dA)
for (i in 2:NS) A[i,i-1]=dA[i-1]
A[1,NS]=2*dA[NS]
out2=lsoda(y=ny0,times=c(8,12,16,24,48),fderiv12, parms=A, rtol=1e-4, atol= 1e-4)
outs=data.frame(rbind(out1,out2))[-3,];#outs
PP=outs[,2:(NS+1)]
Nt=apply(PP,1,sum)
PP=PP/Nt
PP
G1=apply(PP[1:nG1],1,sum)
S=apply(PP[(nG1+1):(nG1+nS)],1,sum)
G2M=apply(PP[(nG1+nS+1):NS],1,sum)
tab=cbind(G1,S,G2M)
rownames(tab)<-outs$time
tab
apply(tab,1,sum)
x=H[[1]]$mids  # these are all the same since br was set in hist
windows(width = 9, height = 5) 
par(mfrow=c(2,4))
for (tt in 1:7)
{
Ys=rep(0,length(x))
for (i in 1:nS) Ys=Ys+PP[tt,nG1+i]*dnorm(x,mean=mid1[tt]+(mid2[tt]-mid1[tt])*i/(nS+1),sd=sig)
Y=G1[tt]*dnorm(x,mean=mid1[tt],sd=sig)+G2M[tt]*dnorm(x,mean=mid2[tt],sd=sig)+Ys
plot(x,Y,xlab="DNA Content",ylab="Probability Density",type="l",main=paste(timept[tt],"h  ",sep=""))
points(x,H[[tt]]$density,pch=1)
}
plot.new()
text(x=.4,y=.95,labels="Yuji Seo's Data")
text(x=.4,y=.85,labels=paste("k=",signif(k,2),sep=""))
text(x=.4,y=.75,labels=paste("sig=",signif(sig,2),sep=""))
text(x=.4,y=.65,labels=paste("kdmod=",signif(kdmod,2),sep=""))
text(x=.4,y=.55,labels=paste("ksmod=",signif(ksmod,2),sep=""))
text(x=.4,y=.42,labels=paste("G1=",nG1,"; S=",nS,"; G2=",nG2,"; M=",nM,sep=""))
#text(x=.4,y=.35,labels=paste("G2=",nG2,"; M=",nM,sep=""))
text(x=.4,y=.30,labels="at t=0: G1-S release")
text(x=.4,y=.21,labels=paste("Tau1=",signif(tau1,2)," h",sep=""))
text(x=.4,y=.09,labels="at t=8: serum starve")
text(x=.4,y=.00,labels=paste("Tau2=",signif(tau2,2)," h",sep=""))
par(mfrow=c(1,1))

